@@ -1,26 +1,29 @@
-# Copyright (c) 2014-2016, 2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2016, 2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Francis Charette Migneault <francis.charette.migneault@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 
 """Astroid hooks for six module."""
 
 from textwrap import dedent
 
-from astroid import MANAGER, register_module_extender
+from astroid import MANAGER, nodes, register_module_extender
 from astroid.builder import AstroidBuilder
 from astroid.exceptions import (
     AstroidBuildingError,
-    InferenceError,
     AttributeInferenceError,
+    InferenceError,
 )
-from astroid import nodes
-
 
 SIX_ADD_METACLASS = "six.add_metaclass"
+SIX_WITH_METACLASS = "six.with_metaclass"
 
 
 def _indent(text, prefix, predicate=None):
@@ -50,7 +53,8 @@ input = input
 from sys import intern
 map = map
 range = range
-from imp import reload as reload_module
+from importlib import reload
+reload_module = lambda module: reload(module)
 from functools import reduce
 from shlex import quote as shlex_quote
 from io import StringIO
@@ -144,8 +148,8 @@ def _six_fail_hook(modname):
         attribute = modname[start_index:].lstrip(".").replace(".", "_")
         try:
             import_attr = module.getattr(attribute)[0]
-        except AttributeInferenceError:
-            raise AstroidBuildingError(modname=modname)
+        except AttributeInferenceError as exc:
+            raise AstroidBuildingError(modname=modname) from exc
         if isinstance(import_attr, nodes.Import):
             submodule = MANAGER.ast_from_module_name(import_attr.names[0][0])
             return submodule
@@ -166,7 +170,7 @@ def _looks_like_decorated_with_six_add_metaclass(node):
     return False
 
 
-def transform_six_add_metaclass(node):
+def transform_six_add_metaclass(node):  # pylint: disable=inconsistent-return-statements
     """Check if the given class node is decorated with *six.add_metaclass*
 
     If so, inject its argument as the metaclass of the underlying class.
@@ -186,6 +190,39 @@ def transform_six_add_metaclass(node):
             metaclass = decorator.args[0]
             node._metaclass = metaclass
             return node
+    return
+
+
+def _looks_like_nested_from_six_with_metaclass(node):
+    if len(node.bases) != 1:
+        return False
+    base = node.bases[0]
+    if not isinstance(base, nodes.Call):
+        return False
+    try:
+        if hasattr(base.func, "expr"):
+            # format when explicit 'six.with_metaclass' is used
+            mod = base.func.expr.name
+            func = base.func.attrname
+            func = f"{mod}.{func}"
+        else:
+            # format when 'with_metaclass' is used directly (local import from six)
+            # check reference module to avoid 'with_metaclass' name clashes
+            mod = base.parent.parent
+            import_from = mod.locals["with_metaclass"][0]
+            func = f"{import_from.modname}.{base.func.name}"
+    except (AttributeError, KeyError, IndexError):
+        return False
+    return func == SIX_WITH_METACLASS
+
+
+def transform_six_with_metaclass(node):
+    """Check if the given class node is defined with *six.with_metaclass*
+
+    If so, inject its argument as the metaclass of the underlying class.
+    """
+    call = node.bases[0]
+    node._metaclass = call.args[0]
 
 
 register_module_extender(MANAGER, "six", six_moves_transform)
@@ -198,3 +235,8 @@ MANAGER.register_transform(
     transform_six_add_metaclass,
     _looks_like_decorated_with_six_add_metaclass,
 )
+MANAGER.register_transform(
+    nodes.ClassDef,
+    transform_six_with_metaclass,
+    _looks_like_nested_from_six_with_metaclass,
+)
